import json
import requests

WEAPON_LIST = ["Sword", "Axe", "Dagger", "Staff", "Bow", "Wand", "Mace"]
def slot_sub(key,value):
    if any(item in value for item in WEAPON_LIST):
        value = "Weapon"
    return [key,value]

print("Fetching basetype info . . .",end=" ")
item_base_types_url = "https://raw.githubusercontent.com/brather1ng/RePoE/master/RePoE/data/base_items.json"
r = requests.get(item_base_types_url)
base_type_dict = dict(json.loads(r.content))
BASE_TYPES = dict([[base_type_dict[base_type]["name"],base_type_dict[base_type]["item_class"]] for base_type in base_type_dict if base_type_dict[base_type]["domain"] == "item"])
SLOT_LOOKUP = dict([slot_sub(k,v) for k,v in BASE_TYPES.items()])
print("done")


if __name__ == "__main__":
    print(SLOT_LOOKUP)
    print(base_type_dict)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_gui.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(718, 357)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.login_link = QtWidgets.QCommandLinkButton(self.centralwidget)
        self.login_link.setGeometry(QtCore.QRect(0, -5, 261, 51))
        font = QtGui.QFont()
        font.setFamily("Segoe UI")
        font.setPointSize(16)
        font.setBold(False)
        font.setWeight(50)
        self.login_link.setFont(font)
        self.login_link.setAutoFillBackground(False)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("../img/poe.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.login_link.setIcon(icon)
        self.login_link.setIconSize(QtCore.QSize(40, 40))
        self.login_link.setAutoRepeatDelay(1000)
        self.login_link.setObjectName("login_link")
        self.select_tab = QtWidgets.QComboBox(self.centralwidget)
        self.select_tab.setGeometry(QtCore.QRect(90, 110, 241, 22))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.select_tab.setFont(font)
        self.select_tab.setEditable(True)
        self.select_tab.setObjectName("select_tab")
        self.select_league = QtWidgets.QComboBox(self.centralwidget)
        self.select_league.setGeometry(QtCore.QRect(90, 80, 241, 22))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.select_league.setFont(font)
        self.select_league.setEditable(True)
        self.select_league.setObjectName("select_league")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setGeometry(QtCore.QRect(10, 80, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setGeometry(QtCore.QRect(10, 110, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.count_report_string = QtWidgets.QLabel(self.centralwidget)
        self.count_report_string.setGeometry(QtCore.QRect(10, 280, 701, 41))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(14)
        self.count_report_string.setFont(font)
        self.count_report_string.setObjectName("count_report_string")
        self.refresh_link = QtWidgets.QCommandLinkButton(self.centralwidget)
        self.refresh_link.setGeometry(QtCore.QRect(250, 0, 91, 41))
        font = QtGui.QFont()
        font.setFamily("Segoe UI")
        font.setPointSize(16)
        font.setBold(False)
        font.setWeight(50)
        self.refresh_link.setFont(font)
        self.refresh_link.setAutoFillBackground(False)
        icon = QtGui.QIcon.fromTheme("refresh")
        self.refresh_link.setIcon(icon)
        self.refresh_link.setIconSize(QtCore.QSize(40, 40))
        self.refresh_link.setAutoRepeatDelay(1000)
        self.refresh_link.setObjectName("refresh_link")
        self.count_weapons = QtWidgets.QProgressBar(self.centralwidget)
        self.count_weapons.setGeometry(QtCore.QRect(440, 10, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_weapons.setFont(font)
        self.count_weapons.setProperty("value", 0)
        self.count_weapons.setObjectName("count_weapons")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setGeometry(QtCore.QRect(360, 10, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.count_helms = QtWidgets.QProgressBar(self.centralwidget)
        self.count_helms.setGeometry(QtCore.QRect(440, 40, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_helms.setFont(font)
        self.count_helms.setProperty("value", 0)
        self.count_helms.setObjectName("count_helms")
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setGeometry(QtCore.QRect(360, 40, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.label_5 = QtWidgets.QLabel(self.centralwidget)
        self.label_5.setGeometry(QtCore.QRect(360, 100, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.count_bodies = QtWidgets.QProgressBar(self.centralwidget)
        self.count_bodies.setGeometry(QtCore.QRect(440, 70, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_bodies.setFont(font)
        self.count_bodies.setProperty("value", 0)
        self.count_bodies.setObjectName("count_bodies")
        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setGeometry(QtCore.QRect(360, 70, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.count_legs = QtWidgets.QProgressBar(self.centralwidget)
        self.count_legs.setGeometry(QtCore.QRect(440, 100, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_legs.setFont(font)
        self.count_legs.setProperty("value", 0)
        self.count_legs.setObjectName("count_legs")
        self.label_7 = QtWidgets.QLabel(self.centralwidget)
        self.label_7.setGeometry(QtCore.QRect(360, 160, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.count_amulets = QtWidgets.QProgressBar(self.centralwidget)
        self.count_amulets.setGeometry(QtCore.QRect(440, 220, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_amulets.setFont(font)
        self.count_amulets.setProperty("value", 0)
        self.count_amulets.setObjectName("count_amulets")
        self.label_8 = QtWidgets.QLabel(self.centralwidget)
        self.label_8.setGeometry(QtCore.QRect(360, 190, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.count_boots = QtWidgets.QProgressBar(self.centralwidget)
        self.count_boots.setGeometry(QtCore.QRect(440, 130, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_boots.setFont(font)
        self.count_boots.setProperty("value", 0)
        self.count_boots.setObjectName("count_boots")
        self.label_9 = QtWidgets.QLabel(self.centralwidget)
        self.label_9.setGeometry(QtCore.QRect(360, 130, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.count_gloves = QtWidgets.QProgressBar(self.centralwidget)
        self.count_gloves.setGeometry(QtCore.QRect(440, 160, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_gloves.setFont(font)
        self.count_gloves.setProperty("value", 0)
        self.count_gloves.setObjectName("count_gloves")
        self.count_belts = QtWidgets.QProgressBar(self.centralwidget)
        self.count_belts.setGeometry(QtCore.QRect(440, 190, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_belts.setFont(font)
        self.count_belts.setProperty("value", 0)
        self.count_belts.setObjectName("count_belts")
        self.label_10 = QtWidgets.QLabel(self.centralwidget)
        self.label_10.setGeometry(QtCore.QRect(360, 220, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.label_12 = QtWidgets.QLabel(self.centralwidget)
        self.label_12.setGeometry(QtCore.QRect(360, 250, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.count_rings = QtWidgets.QProgressBar(self.centralwidget)
        self.count_rings.setGeometry(QtCore.QRect(440, 250, 271, 23))
        font = QtGui.QFont()
        font.setFamily("Bahnschrift")
        font.setPointSize(10)
        self.count_rings.setFont(font)
        self.count_rings.setProperty("value", 0)
        self.count_rings.setObjectName("count_rings")
        self.sets_target = QtWidgets.QSpinBox(self.centralwidget)
        self.sets_target.setGeometry(QtCore.QRect(280, 200, 51, 22))
        self.sets_target.setProperty("value", 8)
        self.sets_target.setObjectName("sets_target")
        self.label_11 = QtWidgets.QLabel(self.centralwidget)
        self.label_11.setGeometry(QtCore.QRect(200, 200, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.item_ = QtWidgets.QLabel(self.centralwidget)
        self.item_.setGeometry(QtCore.QRect(10, 140, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.item_.setFont(font)
        self.item_.setObjectName("item_")
        self.item_1 = QtWidgets.QLabel(self.centralwidget)
        self.item_1.setEnabled(False)
        self.item_1.setGeometry(QtCore.QRect(10, 170, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.item_1.setFont(font)
        self.item_1.setObjectName("item_1")
        self.filter_mode = QtWidgets.QComboBox(self.centralwidget)
        self.filter_mode.setEnabled(False)
        self.filter_mode.setGeometry(QtCore.QRect(110, 170, 221, 22))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.filter_mode.setFont(font)
        self.filter_mode.setObjectName("filter_mode")
        self.item_2 = QtWidgets.QLabel(self.centralwidget)
        self.item_2.setEnabled(False)
        self.item_2.setGeometry(QtCore.QRect(10, 200, 111, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.item_2.setFont(font)
        self.item_2.setObjectName("item_2")
        self.recipe_chaos = QtWidgets.QRadioButton(self.centralwidget)
        self.recipe_chaos.setEnabled(False)
        self.recipe_chaos.setGeometry(QtCore.QRect(70, 200, 91, 17))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(10)
        self.recipe_chaos.setFont(font)
        self.recipe_chaos.setChecked(True)
        self.recipe_chaos.setObjectName("recipe_chaos")
        self.recipe_type = QtWidgets.QButtonGroup(MainWindow)
        self.recipe_type.setObjectName("recipe_type")
        self.recipe_type.addButton(self.recipe_chaos)
        self.recipe_regal = QtWidgets.QRadioButton(self.centralwidget)
        self.recipe_regal.setEnabled(False)
        self.recipe_regal.setGeometry(QtCore.QRect(70, 220, 91, 17))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(10)
        self.recipe_regal.setFont(font)
        self.recipe_regal.setObjectName("recipe_regal")
        self.recipe_type.addButton(self.recipe_regal)
        self.item_filter_browse = QtWidgets.QPushButton(self.centralwidget)
        self.item_filter_browse.setGeometry(QtCore.QRect(90, 140, 241, 23))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(10)
        self.item_filter_browse.setFont(font)
        self.item_filter_browse.setObjectName("item_filter_browse")
        self.client_secret_input = QtWidgets.QLineEdit(self.centralwidget)
        self.client_secret_input.setGeometry(QtCore.QRect(90, 50, 241, 20))
        self.client_secret_input.setObjectName("client_secret_input")
        self.label_13 = QtWidgets.QLabel(self.centralwidget)
        self.label_13.setGeometry(QtCore.QRect(10, 50, 71, 21))
        font = QtGui.QFont()
        font.setFamily("Roboto")
        font.setPointSize(12)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 718, 21))
        self.menubar.setObjectName("menubar")
        self.menuLinks = QtWidgets.QMenu(self.menubar)
        self.menuLinks.setObjectName("menuLinks")
        self.menuSettings = QtWidgets.QMenu(self.menubar)
        self.menuSettings.setObjectName("menuSettings")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionFilterblade_xyz = QtWidgets.QAction(MainWindow)
        self.actionFilterblade_xyz.setObjectName("actionFilterblade_xyz")
        self.actionChipy_dev = QtWidgets.QAction(MainWindow)
        self.actionChipy_dev.setObjectName("actionChipy_dev")
        self.actionPathOfExile_com = QtWidgets.QAction(MainWindow)
        self.actionPathOfExile_com.setObjectName("actionPathOfExile_com")
        self.actionPathOfBuilding_com = QtWidgets.QAction(MainWindow)
        self.actionPathOfBuilding_com.setObjectName("actionPathOfBuilding_com")
        self.actionCraftOfExile_com = QtWidgets.QAction(MainWindow)
        self.actionCraftOfExile_com.setObjectName("actionCraftOfExile_com")
        self.actionMap_RegEx = QtWidgets.QAction(MainWindow)
        self.actionMap_RegEx.setObjectName("actionMap_RegEx")
        self.actionVorici_Calculator = QtWidgets.QAction(MainWindow)
        self.actionVorici_Calculator.setObjectName("actionVorici_Calculator")
        self.actionPoE_Ninja = QtWidgets.QAction(MainWindow)
        self.actionPoE_Ninja.setObjectName("actionPoE_Ninja")
        self.actionPoE_Lab = QtWidgets.QAction(MainWindow)
        self.actionPoE_Lab.setObjectName("actionPoE_Lab")
        self.actionAwakened_PoE_Trade = QtWidgets.QAction(MainWindow)
        self.actionAwakened_PoE_Trade.setObjectName("actionAwakened_PoE_Trade")
        self.actionInput_ClientSecret = QtWidgets.QAction(MainWindow)
        self.actionInput_ClientSecret.setObjectName("actionInput_ClientSecret")
        self.menuLinks.addAction(self.actionChipy_dev)
        self.menuLinks.addSeparator()
        self.menuLinks.addAction(self.actionPathOfExile_com)
        self.menuLinks.addAction(self.actionPathOfBuilding_com)
        self.menuLinks.addAction(self.actionCraftOfExile_com)
        self.menuLinks.addAction(self.actionFilterblade_xyz)
        self.menuLinks.addAction(self.actionMap_RegEx)
        self.menuLinks.addAction(self.actionVorici_Calculator)
        self.menuLinks.addAction(self.actionPoE_Ninja)
        self.menuLinks.addAction(self.actionPoE_Lab)
        self.menuLinks.addAction(self.actionAwakened_PoE_Trade)
        self.menuSettings.addAction(self.actionInput_ClientSecret)
        self.menubar.addAction(self.menuSettings.menuAction())
        self.menubar.addAction(self.menuLinks.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.login_link.setText(_translate("MainWindow", "Log In?"))
        self.label.setText(_translate("MainWindow", "League:"))
        self.label_2.setText(_translate("MainWindow", "Tab:"))
        self.count_report_string.setText(_translate("MainWindow", "TextLabel"))
        self.refresh_link.setText(_translate("MainWindow", "Refresh"))
        self.label_3.setText(_translate("MainWindow", "Weapons"))
        self.label_4.setText(_translate("MainWindow", "Helms"))
        self.label_5.setText(_translate("MainWindow", "Legs"))
        self.label_6.setText(_translate("MainWindow", "Bodies"))
        self.label_7.setText(_translate("MainWindow", "Gloves"))
        self.label_8.setText(_translate("MainWindow", "Belts"))
        self.label_9.setText(_translate("MainWindow", "Boots"))
        self.label_10.setText(_translate("MainWindow", "Amulets"))
        self.label_12.setText(_translate("MainWindow", "Rings"))
        self.label_11.setText(_translate("MainWindow", "Set goal:"))
        self.item_.setText(_translate("MainWindow", "Item Fitler:"))
        self.item_1.setText(_translate("MainWindow", "Filter Mode:"))
        self.item_2.setText(_translate("MainWindow", "Recipe:"))
        self.recipe_chaos.setText(_translate("MainWindow", "Chaos (i60+)"))
        self.recipe_regal.setText(_translate("MainWindow", "Regal (i70+)"))
        self.item_filter_browse.setText(_translate("MainWindow", "Browse"))
        self.label_13.setText(_translate("MainWindow", "SECRET:"))
        self.menuLinks.setTitle(_translate("MainWindow", "Links"))
        self.menuSettings.setTitle(_translate("MainWindow", "Settings"))
        self.actionFilterblade_xyz.setText(_translate("MainWindow", "Filterblade.xyz"))
        self.actionChipy_dev.setText(_translate("MainWindow", "Chipy.dev"))
        self.actionPathOfExile_com.setText(_translate("MainWindow", "PathOfExile.com"))
        self.actionPathOfBuilding_com.setText(_translate("MainWindow", "PathOfBuilding.com"))
        self.actionCraftOfExile_com.setText(_translate("MainWindow", "CraftOfExile.com"))
        self.actionMap_RegEx.setText(_translate("MainWindow", "Map RegEx"))
        self.actionVorici_Calculator.setText(_translate("MainWindow", "Vorici Calculator"))
        self.actionPoE_Ninja.setText(_translate("MainWindow", "PoE Ninja"))
        self.actionPoE_Lab.setText(_translate("MainWindow", "PoE Lab"))
        self.actionAwakened_PoE_Trade.setText(_translate("MainWindow", "Awakened PoE Trade"))
        self.actionInput_ClientSecret.setText(_translate("MainWindow", "Input ClientSecret"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())



import typing
import datetime
import os
import sys
import time
import webbrowser
from PyQt5.QtWidgets import QMainWindow, QFileDialog, QApplication
from PyQt5 import QtGui
from PyQt5.QtCore import QTimer
import qt.main_gui
import ctypes
import poepy
import user_info

# type checking block (AND RUFF INFO)
# https://www.youtube.com/watch?v=bcAqceZkZRQ
if typing.TYPE_CHECKING:
    ...

# set statics
ASYNC_INTERVAL_MS = 2000

api:poepy.PoeApiHandler
parser:poepy.DataParser

# variables for searching log files to detect new zone
modified = 0
previous = 0

class AsyncMainWindow(QMainWindow):
    log_timer = QTimer()
    def __init__(self):
        super().__init__()
        self.init_async()

    def init_async(self):
        print("Initializing . . . ", end="")
        self.log_timer.timeout.connect(async_two)
        self.log_timer.start(ASYNC_INTERVAL_MS)
        print("Started")

def timed_try_wrapper(function):
    """Simple timing wrapper that also includes a TRY loop
    Args:
        function (Func): Returns the provided function wrapped with Time and Try
    """
    def wrapper(*args, **kwargs):
        start = time.time()
        try:
            result = function(*args, **kwargs)
            end = time.time()
            print(function.__name__, "=>RunTime:",end-start)
            return result
        except Exception as e:
            end = time.time()
            print(function.__name__, "=>FAILED:",end-start, e)
            result = False
    return wrapper    

def apply_ui_defaults():
    global gui_main
    head, filter_name = os.path.split(user_info.cfg["form"]["filter_name"])
    gui_main.item_filter_browse.setText(filter_name)
    gui_main.client_secret_input.setText(user_info.get("api","client_secret"))

def apply_ui_connections():
    """Overlay that connects up the GUI so that we can modularly replace the gui.py from QT5
    https://www.geeksforgeeks.org/function-wrappers-in-python/
    Args:
        gui_obj (gui.Ui_MainWindow): Main window GUI object
    """
    global gui_main, MainWindow

    # set window Icons
    app.setWindowIcon(QtGui.QIcon('./cpct/cpct\img\ChipyLogo.png'))
    MainWindow.setWindowTitle("Chipy's PoE Chaos Tool")
    
    # set login icon (this is to fix the image path issue)
    icon = QtGui.QIcon()
    icon.addPixmap(QtGui.QPixmap("./cpct/cpct/img/poe.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
    gui_main.login_link.setIcon(icon)

    # # link menus
    gui_main.actionChipy_dev.triggered.connect(lambda: webbrowser.open("www.chipy.dev/me.html"))
    gui_main.actionFilterblade_xyz.triggered.connect(lambda: webbrowser.open("https://www.filterblade.xyz/") )
    gui_main.actionCraftOfExile_com.triggered.connect(lambda: webbrowser.open("https://www.craftofexile.com/en/") )
    gui_main.actionMap_RegEx.triggered.connect(lambda: webbrowser.open("https://poe.re/#/maps") )
    gui_main.actionPathOfBuilding_com.triggered.connect(lambda: webbrowser.open("https://pathofbuilding.community/") )
    gui_main.actionPathOfExile_com.triggered.connect(lambda: webbrowser.open("https://www.pathofexile.com") )
    gui_main.actionPoE_Lab.triggered.connect(lambda: webbrowser.open("https://www.poelab.com/") )
    gui_main.actionPoE_Ninja.triggered.connect(lambda: webbrowser.open("https://www.poe.ninja") )
    gui_main.actionVorici_Calculator.triggered.connect(lambda: webbrowser.open("https://siveran.github.io/calc.html") )
    gui_main.actionAwakened_PoE_Trade.triggered.connect(lambda: webbrowser.open("https://github.com/SnosMe/awakened-poe-trade") )
    
    #ClientSecrect Menu
    # gui_main.actionInput_ClientSecret.triggered.connect(lambda: receive_client_secret(gui_main) )

    # # link buttons
    gui_main.login_link.clicked.connect(lambda: action_login_link(gui_main))
    gui_main.refresh_link.clicked.connect(lambda: update_unid_counts(gui_main, True))
    gui_main.item_filter_browse.clicked.connect(lambda: open_browser())

    # Link ComboBoxes
    gui_main.select_league.currentIndexChanged.connect(lambda: action_set_league(gui_main))
    gui_main.select_tab.currentIndexChanged.connect(lambda: action_set_tab(gui_main))

    # Link Text
    gui_main.client_secret_input.textChanged.connect(lambda: receive_client_secret(gui_main))

# @timed_try_wrapper
def action_login_link(gui):
    global api, parser, gui_main
    api = poepy.PoeApiHandler(client_id=user_info.cfg["api"]["CLIENT_ID"],
                                    client_secret=user_info.cfg["api"]["CLIENT_SECRET"],
                                    scope=user_info.cfg["api"]["SCOPE"],
                                    uri=user_info.cfg["api"]["REDIRECT_URI"],
                                    manual_token=user_info.cfg["api"]["TOKEN"]
                                    )
    parser = poepy.DataParser(api_handler = api)

    # save any token changes
    user_info.set("api","TOKEN", api.token)
    user_info.set("form","username", parser.get_username())
    gui_main.client_secret_input.isEnabled = False

    # set login name
    gui.login_link.setText(user_info.get("form","username"))
    gui.login_link.setDisabled(True)
    gui.select_league.setCurrentText( user_info.get("form","league"))
    gui.select_tab.setCurrentText( user_info.get("form","tab"))

    # continue the loading chain
    action_load_leagues(gui)

def action_load_leagues(gui):
    global parser
    leagues = parser.get_leagues()
    # clear the box and repop
    gui.select_league.clear()
    gui.select_tab.clear()
    gui.select_league.addItems(leagues)
    # # set previous league
    # gui_main.select_league.setCurrentText( user_info.cfg["form"]["league"])
    
def action_set_league(gui):
    league = gui.select_league.currentText()
    user_info.cfg["form"]["league"] = gui.select_league.currentText()
    user_info.save()    
    action_load_tabs(gui, league)

def action_load_tabs(gui, league):
    global parser, gui_main
    tabs = parser.get_tab_names(league).keys()
    # clear the box and repop
    gui.select_tab.clear()
    gui.select_tab.addItems(tabs)
    
def action_set_tab(gui, force_recache:bool=False):
    global parser, gui_main
    user_info.cfg["form"]["tab"] = gui.select_tab.currentText()
    user_info.save()
  
def update_unid_counts(gui, force_recache:bool=False):
    global parser, gui_main
    league_of_interest = gui.select_league.currentText()
    try:
        # tab_of_interes
        tabs_of_interest = poepy.validate_tab(parser, league_of_interest, gui.select_tab.currentText())

        # filter for unid
        list_of_items_unidentified = parser.filter_identified(parser.get_items(tabs_of_interest, league_of_interest, force_recache))
        
        # loop and count unids
        count = poepy.count_slots(parser, list_of_items_unidentified)
        target = gui.sets_target.value()
        # str_count = str(count.items())
        # gui.count_report_string.setText(str_count)
        multiplier = 100//target
        
        # set GUI element values
        gui_main.count_weapons.setValue(count["Weapon"]*multiplier)
        gui_main.count_helms.setValue(count["Helmet"]*multiplier)
        gui_main.count_bodies.setValue(count["Body"]*multiplier)
        gui_main.count_boots.setValue(count["Boots"]*multiplier)
        gui_main.count_gloves.setValue(count["Gloves"]*multiplier)
        gui_main.count_belts.setValue(count["Belt"]*multiplier)
        gui_main.count_amulets.setValue(count["Amulet"]*multiplier)
        gui_main.count_rings.setValue((count["Ring"]*multiplier)/2)
    except Exception as e:
        gui.count_report_string.setText(str(e))

def async_two():
    # Entry point to secondary exec chain
    log_search()

def log_search():
    global modified, previous, gui_main
    # 2023/03/30 09:11     
    # 2023/03/30 09:26:41 1117798968 cffb0734 [INFO Client 31504] : You have entered Aspirants' Plaza.     
    snippet = " : You have entered"
    path = "C:\Program Files (x86)\Grinding Gear Games\Path of Exile\logs\Client.txt"
    modified = os.path.getmtime(path)
    if modified > previous:
        previous = modified
        print("Last modified: %s" % time.ctime(modified))
        gui_main.count_report_string.setText("Reading...")
        stamp = datetime.datetime.now().strftime("%Y/%m/%d %H:%M")
        with open(path, "r", encoding="utf-8") as file:
            for line in file:
                if stamp in line and snippet in line:
                        print(line)
                        gui_main.count_report_string.setText(line[78:])
                        update_unid_counts(gui_main)
                        return
        gui_main.count_report_string.setText("Reading... Done")

def open_browser():
    global MainWindow
    #C:\Users\chipy\Documents\My Games\Path of Exile\
    file_dialog = QFileDialog(MainWindow)
    file_dialog.setFileMode(QFileDialog.AnyFile)
    file_dialog.setNameFilter("Item Filter (*.filter)")
    file_dialog.setDirectory(user_info.cfg["form"]["filter_dir"])
    
    if file_dialog.exec_():
        user_info.cfg["form"]["filter_name"] = file_dialog.selectedFiles()[0]
        user_info.save()

def receive_client_secret(gui):
    global gui_main
    user_info.set("api","client_secret",gui_main.client_secret_input.text())


if __name__ == "__main__":
    # required for Windows to recognize a Python script as it's own applications and thus have a unique Taskbar Icon
    # https://stackoverflow.com/questions/1551605/how-to-set-applications-taskbar-icon-in-windows-7/1552105#1552105
    myappid = u'chipy.PoE.chaos.tool' # arbitrary string
    ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

    # build main GUI
    app = QApplication(sys.argv)
    MainWindow = AsyncMainWindow()
    MainWindow.show()
    gui_main = qt.main_gui.Ui_MainWindow()
    gui_main.setupUi(MainWindow)

    # Modify the gui with connections and links
    apply_ui_connections()  # here we modify actions to the GUI
    apply_ui_defaults()  # set default values for the form when it's made

    # run app as the last thing in the script
    sys.exit(app.exec_())
  


import asyncio
import hashlib
import json
import random
import webbrowser
from urllib.parse import parse_qs, urlparse

import requests
import websockets

from base_types import SLOT_LOOKUP

HEADER_USER_AGENT ={"User-Agent": "OAuth chipytools/0.0.1 (Contact: contact@chipy.dev)"}
HEADER_TYPE = {"Content-Type": "application/x-www-form-urlencoded"}
URL_AUTH = r"https://www.pathofexile.com/oauth/authorize"
URL_TOKEN = r"https://www.pathofexile.com/oauth/token"
API_ENDPOINT = r"https://api.pathofexile.com/"
API_PROFILE = API_ENDPOINT+"profile"
API_CHARACTER = API_ENDPOINT+"character"
API_LEAGUE = API_ENDPOINT+"league"
API_STASH = API_ENDPOINT+"stash/"

DEPTH_ITEMS = 2
DEPTH_STASH_NAMES = 1

"""
PROCESS
- Connection handler
- - Calls
- - - Output data
- Data handler
- - processer
- - 
"""

class PoeApiHandler():
    def __init__(self, 
                 client_id, 
                 client_secret, 
                 uri, 
                 scope="account:profile", 
                 force_re_auth:bool=False, 
                 manual_token:str=None):
        print("Building variables . . . ",end=" ")
        self.id = client_id
        self.secret = client_secret
        self.uri = uri
        self.state = hashlib.sha256(str(random.randint(2341,8599)).encode()).hexdigest()
        self.scope = scope
        self.code = ""
        self.token = ""
        self.headers = {**HEADER_TYPE,**HEADER_USER_AGENT}  
        print("done")

        if manual_token:
            self._update_header_token(manual_token)
        self._authenticate(force_re_auth)
        print( "done")
        

    async def parse(self, url):
        
        print("Parsing . . . ", end="")
        self.url_dict = urlparse(url)
        queries = parse_qs(self.url_dict[4])  
        if "error" in queries:
            print("PROBLEM WITH OAUTH REPLY:")        
            print(queries)
            exit()
        self.state=queries["state"][0]
        self.code=queries["code"][0]

    async def echo(self, websocket, path):
        async for message in websocket:
            await websocket.send(message)
            print("done")
            self.msg = message
            await self.parse(self.msg)
            self._exit.set_result(None)
            print("done")
                
    async def echo_server(self):
        async with websockets.serve(self.echo, '127.0.0.1', 32111):
            await self._exit       

    def _update_header_token(self,token_to_update_with=False):
        if token_to_update_with:
            self.token = token_to_update_with
        self.headers = {"Authorization": "Bearer "+self.token, 
                        **HEADER_USER_AGENT}
        print("LOADING TOKEN:",self.token)
        
    def is_request_successful(self, request_code:str):
        if "200" in str(request_code):
            return 1
        if "400" in str(request_code):
            print("Bad Request: Please check user config for ClientSecret")
        return False
        
    def _still_authenticated(self):
        if self.is_request_successful(self.get_stash("standard")):
            return True
        print("FAILED with cached token")
        return False
            
    def _authenticate(self, force_re_auth=False):
        print("Authenticating . . . ",end="")
        # test status
        if self._still_authenticated() and not force_re_auth:
            return
        
        try:
            if self.renew_auth_token():
                return
        except Exception as e:
            print("AuthRenewError:",e)
            pass 
        # Building URL for permission request
        # print("Building request ...")
        client_str = '?client_id='+self.id
        response_type = '&response_type=code'
        scope_str = '&scope='+self.scope
        state_str = '&state='+self.state
        redir = '&redirect_uri='+self.uri

        # print("Initializing OAuth2 ...")
        webbrowser.open(URL_AUTH+client_str+response_type+scope_str+state_str+redir)
        # print(URL_AUTH+client_str+response_type+scope_str+state_str+redir)

        # Start the async loop
        print("Waiting for approval . . . ",end="")
        self.loop = asyncio.get_event_loop()
        self._exit = asyncio.Future() 
        self.loop.run_until_complete(self.echo_server())

        # build variables for the exchange
        self.token_data = { "client_id":self.id,
                            "client_secret": self.secret,
                            "grant_type":"authorization_code",
                            "code":self.code,
                            "redirect_uri":self.uri,
                            "scope":self.scope}
        
        self.headers = {**HEADER_TYPE,**HEADER_USER_AGENT}       
         
        # Make the code -> token exchange
        request = requests.post(URL_TOKEN, data=self.token_data, headers=self.headers)
        # request.raise_for_status()

        # check for issues:
        if self.is_request_successful(request):
            self.auth_reply_raw = json.loads(request.content)
            self._update_header_token(self.auth_reply_raw["access_token"])
        
    def renew_auth_token(self):
        self.body_refresh = {"client_secret":self.secret,
                             "grant_type":"refresh_token",
                             "refresh_token":self.auth_reply_raw["refresh_token"]}
        request = requests.post(URL_TOKEN, data=self.token_data, headers=self.headers)
        request.raise_for_status()

        self.auth_reply_raw = json.loads(request.content)
        self._update_header_token(self.auth_reply_raw["access_token"])
        return self._still_authenticated()

    def get_stash(self, league) -> requests.Response:
        r = requests.get(API_STASH+league, headers=self.headers)
        if self.is_request_successful(r):
            return r
        return False    
    
    def get_profile(self) -> requests.Response:
        r = requests.get(API_PROFILE, headers=self.headers)
        if self.is_request_successful(r):
            return r
        return False
    
    def get_tab(self, league:str, stash_id:str) -> requests.Response:
        r = requests.get(API_STASH+league+"/"+stash_id, headers=self.headers)
        if self.is_request_successful(r):
            return r
        return False    
    
    def get_leagues(self) -> requests.Response:
        # this is a service scope request so it's a bit different
        # (not private so no auth needed)
        #https://www.pathofexile.com/developer/docs/authorization#client_credentials
        # first get a token
        data = {"client_id":self.id,
                "client_secret": self.secret,
                "grant_type":"client_credentials",
                "scope":"service:leagues"}
        r = requests.post(URL_TOKEN, data=data, headers={**HEADER_TYPE,
                                                         **HEADER_USER_AGENT} )
        # adjust a temp header
        temp_header = {**HEADER_TYPE,
                       **HEADER_USER_AGENT,
                       "Authorization":"Bearer "+json.loads(r.content)["access_token"]}
        r = requests.get(API_LEAGUE, headers=temp_header)  
        if self.is_request_successful(r):
            return r
        return False  
    
    def get_characters(self) -> requests.Response:
        r = requests.get(API_CHARACTER, headers=self.headers) 
        if self.is_request_successful(r):
            return r
        return False       

class DataParser():
    def __init__(self, api_handler:PoeApiHandler=None, league="standard") -> None:
        if not api_handler:
            print("API object missing. MAKE sure to use '.new_api_handler(api_handler)'")  
        self.api_handler = api_handler

        self.league = league
        self.cached = {"DEFAULT_VALUE":0}

    def _cache_stash(self, league:str,force_recache:bool=False):
        """Caches the list of tabs in a league's stash |get_leagues(self) -> list:|
            {'id': 'fae1b5d2ef', 
            'name': 'Heist (Remove-only)', 
            'type': 'NormalStash', 
            'index': 0, 
            'metadata': {'colour': '7c5436'}}

        Args:
            league (str): league name as 
        """
        if league+"_stash_response" not in self.cached or force_recache:
            self.cached[league+"_stash_response"] = self.api_handler.get_stash(league)
            self.cached[league+"_stash"] = json.loads(self.cached[league+"_stash_response"].content)["stashes"]
    
    def _parse_tab_names(self, stash:dict, filter_remove_only=True) -> dict:
        result = [[i["name"],i["id"]] for i in stash]
        if filter_remove_only:
            result = [i for i in result if "Remove-only" not in i[0]]
        return dict(result)
    
    def find_tab(self, 
                 search_str:str, 
                 league:str="standard", 
                 all_matches:bool=False) -> tuple:
        """Searches for the provided str in both tab names and IDs

        Args:
            search_str (str): Partial or complete case sensitive string to find
            league (str, optional): League Name string. Defaults to "standard".

        Returns:
            tuple: tab's (name, ID) pair
        """
        # first cache the data we need use
        self._cache_stash(league)

        # set some helper to assist with matching
        last_match = None
        name_match = []
        prioritize_name = len(search_str) != 10

        # search stashes for a match
        for tab in self.cached[league+"_stash"]:
            # check for any partial match
            if search_str in tab["name"] or search_str in tab["id"]:
                # load into variable 
                last_match = (tab["name"],tab["id"])
                # print("something:", last_match)
                # full match return right away
                if search_str == tab["name"] or search_str == tab["id"]:
                    # print("FULLMATCH------------------")
                    return last_match          
                # store name match for priority      
                if prioritize_name and search_str in tab["name"]:
                    # if all_matches:
                    #     name_match.append((tab["name"], tab["id"]))
                    # else:
                    name_match = (tab["name"], tab["id"])
                    # print("nameMatch", name_match)
        if prioritize_name:
            return name_match
        return last_match
       
    def get_tab_names(self, league="standard") -> dict:
        self._cache_stash(league)
        self.cached[league+"_tab_names"] = self._parse_tab_names(stash=self.cached[league+"_stash"])
        return self.cached[league+"_tab_names"]
        
    def _cache_tab(self, league:str, stash_id:str, force_recache:bool=False) -> dict:
        if league+"_"+stash_id not in self.cached or force_recache:
            self.cached[league+"_"+stash_id+"_response"] = self.api_handler.get_tab(league, stash_id)
            raw=json.loads(self.cached[league+"_"+stash_id+"_response"].content)
            # print(type(raw))
            # print(type(raw["stash"]))
            # print(type(raw["stash"]["items"]))
            assert "children" not in raw["stash"]  # assert not a parent/nested tab
            self.cached[league+"_"+stash_id] = raw
        return self.cached[league+"_"+stash_id]

    def _parse_item_names(self, tab:dict) -> list:
        # print(tab)
        # print(type(tab))
        result = [i["name"] for i in tab["stash"]["items"]]
        return result      

    def get_item_names(self,  stash_id:str="52dc1b3814", league="hardcore") -> dict:
        self._cache_tab(league,stash_id)
        self.cached[league+"_"+stash_id+"_item_names"] = self._parse_item_names(self.cached[league+"_"+stash_id])
        return self.cached[league+"_"+stash_id+"_item_names"] 
    
    def get_items(self, 
                  stash_id:str="52dc1b3814", 
                  league="hardcore", 
                  force_recache:bool=False) -> dict:
        # handle when Stash_ID is False
        if isinstance(stash_id, bool):
            return False
        # handle when Stash_ID is the name/ID tuple
        if isinstance(stash_id,tuple) and len(stash_id)==2:
            stash_id=stash_id[1]
        # Handle when you are given a list of StashID
        if isinstance(stash_id,list):
            result_list = []
            for stash in stash_id:
                fetch = self.get_items(stash, league)
                if fetch:
                    result_list+=fetch
            # print(result_list)
            return result_list            

        assert isinstance(stash_id,str) and len(stash_id)==10  # Assert valid stashID 
        self._cache_tab(league,stash_id, force_recache)
        # return self.cached[league+"_"+stash_id]["stash"]["items"]
        try:
            return self.cached[league+"_"+stash_id]["stash"]["items"]
        except KeyError as e:
            print(f"Failed to get stash: {stash_id} [no key 'items' in object] {e}")
            return False
    
    def filter_identified(self, list_of_items:list) -> list:
        return [i for i in list_of_items if i["identified"] is False]



    def _cache_profile(self):
        if "profile" not in self.cached:
            self.cached["profile_response"] = self.api_handler.get_profile()
            self.cached["profile_name"] = json.loads(self.cached["profile_response"].content)["name"]
    
    def get_username(self) -> str:
        self._cache_profile()
        return self.cached["profile_name"]

    def _cache_characters(self):
        if "characters" not in self.cached:
            self.cached["characters_response"] = self.api_handler.get_characters()
            self.cached["characters"] = json.loads(self.cached["characters_response"].content)["characters"]

    def _parse_character_names(self, characters):
        result = [[i["name"],i["league"]] for i in characters]
        return result      
    
    def get_characters(self) -> list:
        self._cache_characters()
        return self._parse_character_names(self.cached["characters"])   

    def _cache_leagues(self):
        if "leagues" not in self.cached:
            self.cached["leagues_response"] = self.api_handler.get_leagues()
            self.cached["leagues"] = json.loads(self.cached["leagues_response"].content)["leagues"]

    def _parse_league_names(self, characters):
        result = [i["id"] for i in characters if i["realm"] == "pc"]
        return result  

    def get_leagues(self) -> list:
        """Base leagues:
            - 'Standard'
            - 'Hardcore'
            - 'SSF Standard'
            - 'SSF Hardcore'
        Returns:
            list: List of active leagues
        """
        self._cache_leagues()
        return self._parse_league_names(self.cached["leagues"])

def validate_league(parser:DataParser, user_input:str=None):
    active_leagues = parser.get_leagues()
    if not user_input:
        print(active_leagues)
        user_input = input("Select League: ").lower()
    for league in active_leagues:
        if user_input in str(league).lower():
            print("League auto-corrected to:",league)
            return league
    return False

def validate_tab(parser:DataParser,
                 league_of_interest:str=None, 
                 user_input:str=None) -> tuple:
    if not user_input:
        print(parser.get_tab_names(league_of_interest))
        user_input = input("Select tab: ")
    if not league_of_interest:
        league_of_interest =validate_league(parser)

    tab = parser.find_tab(user_input, league_of_interest)

    if tab:
        print("League auto-corrected to:",tab)
        return tab
    return False

def count_slots(parser:DataParser, list_of_items:list):
    counts={"Total":0}
    for item in list_of_items:
        slot = SLOT_LOOKUP.get(item["baseType"], "Unknown")
        if slot in counts:
            counts[slot] +=1
        else:
            counts[slot] = 1
        counts["Total"] += 1
    
    return counts



# import private
import configparser
import os

# const
CFG_FILENAME = r"user_settings.cfg"
# CFG info
cfg = configparser.ConfigParser()

def save():
    with open(CFG_FILENAME,"w") as config_file:
        cfg.write(config_file)

def load():
    # Set deafults
    cfg["form"] = {"filter_dir": os.path.expanduser("~/Documents")+"/My Games/Path of Exile/",
                   "filter_name":"Browse To Select",
                   "username":"",
                   "league":"",
                   "tab":""}
    cfg["api"] = {"CLIENT_ID":"chipytools",
                  "CLIENT_SECRET":"AskChipyForThis",
                  "SCOPE":"account:profile account:characters account:stashes account:item_filter",
                  "REDIRECT_URI":"https://chipy.dev/poe_auth.html",
                  "TOKEN":"",
                  }
    cfg.read(CFG_FILENAME)

def get(section:str, key:str) -> str:
    return cfg.get(section, key,fallback="MISSING")

def set(section:str, key:str, value:str):
    cfg[section][key] = value
    save()

# load base info
load()

if __name__ == "__main__":
    print(get("api","client_id"))
    load()
    print(get("api","client_id"))
    save()